.img.fmt.RGB32 <- 0x42475220
.img.fmt.RGB24 <- 0x42475218
.img.fmt.BGR32 <- 0x52474220
.img.fmt.BGR24 <- 0x52474218
.img.fmt.YV12  <- 842094169
.img.fmt.YUY2  <- 0x32595559

run.mplayer <- function(file) {
    system(paste("mplayer -quiet -nosound -vo dl:",system.file("libs",.Platform$r_arch,paste("videoTools",.Platform$dynlib.ext,sep=''),package="videoTools")," ",file,sep=''),wait=FALSE)
}

mplayer <- function(input, FUN, setup.FUN=NULL, format="RGB24", expand=TRUE, port=7922, unix.socket=NULL, save.env=NULL, frames=-1, timeout=10, close=TRUE, resume=FALSE) {
    if (is.character(format)) format <- fourcc2int(format)
    if (!resume) {
      if (!is.null(unix.socket)) port <- 0
      .Call("MP_bind", unix.socket, port, PACKAGE="videoTools")
      if (missing(input)) {
        cat("no input specified, please start mplayer manually now ...\n",paste("mplayer -quiet -nosound -vo dl:",system.file("libs",.Platform$r_arch,paste("videoTools",.Platform$dynlib.ext,sep=''),package="videoTools"),sep=''),"\n\nFor remote feed use VOHOST/VOPORT environment variables when starting mplayer.\n")
      } else {
        if (!is.null(unix.socket))
          Sys.setenv(VOSOCKET=unix.socket)
        else
          Sys.setenv(VOPORT=port)
        if (run.mplayer(input) != 0) {
          .Call("MP_close", PACKAGE="videoTools")
          warning("unable to start mplayer")
          return(-2)
        }
      }
      if (!.Call("MP_accept", timeout, PACKAGE="videoTools")) {
        .Call("MP_close", PACKAGE="videoTools")
        return(-1)
      }
    }

    first <- TRUE
    frame <- 1
    w <- 0
    h <- 0

    if (!is.null(save.env)) save.env$frame <- list()
    if (missing(FUN) || is.null(FUN)) FUN <- function(...) NULL
    if (is.null(setup.FUN)) setup.FUN <- function(...) NULL

    dumpFun <- function(c, r) {
        if (first) { first <<- FALSE; w <<- c[1]; h <<- c[2]; if (!is.null(setup.FUN)) setup.FUN(c[1],c[2]) }
        #cat(sprintf("dumpFun: %08x %d %08x %d %d\n", c[5], c[4], c[3], c[1], c[2]))
            if (c[3]==.img.fmt.RGB32 || c[3]==.img.fmt.RGB24 || c[3]==.img.fmt.BGR32 || c[3]==.img.fmt.BGR24) {
                l <- rawi2double(r,0,w*h*c[4],width=w,channels=c[4])
                if (!is.null(save.env)) save.env$frame[[frame]] <- l
                if (as.integer((c[5]/512)+0.2)%%2)
                    FUN(l[[3]],l[[2]],l[[1]])
                else
                    FUN(l[[1]],l[[2]],l[[3]])
            } else if (c[3]==.img.fmt.YV12) {
                if (expand)
                    FUN(raw2double(r,0,w*h,stride=w),
                        raw2double(r,w*h,w*h/4,stride=w/2,x.exp=2,y.exp=2)-0.5,
                        raw2double(r,w*h*2,w*h/4,stride=w/2,x.exp=2,y.exp=2)-0.5)
                else
                    FUN(raw2double(r,0,w*h,stride=w),
                        raw2double(r,w*h,w*h/4,stride=w/2)-0.5,
                        raw2double(r,w*h*2,w*h/4,stride=w/2)-0.5)
                if (!is.null(save.env)) {
                    if (expand)
                        save.env$frame[[frame]] <- list(raw2double(r,0,w*h,stride=w),
                                                        raw2double(r,w*h,w*h/4,stride=w/2,x.exp=2,y.exp=2)-0.5,
                                                        raw2double(r,w*h*2,w*h/4,stride=w/2,x.exp=2,y.exp=2)-0.5)
                    else
                        save.env$frame[[frame]] <- list(raw2double(r,0,w*h,stride=w),
                                                        raw2double(r,w*h,w*h/4,stride=w/2)-0.5,
                                                        raw2double(r,w*h*2,w*h/4,stride=w/2)-0.5)
                }
            } else if (c[3] ==.img.fmt.YUY2) {
                l <- rawi2double(r,0,w*h*c[4],width=w,channels=c[4])
                if (!is.null(save.env)) save.env$frame[[frame]] <- list(l[[1]],l[[2]]-0.5)
                FUN(l[[1]],l[[2]]-0.5)
            } else {
                FUN(r,w,h,c[3],c[4],c[5])
                if (!is.null(save.env)) save.env$frame[[frame]] <- list(data=r, width=w, height=h, format=c[3], channels=c[4], flags=c[5])
            }
            frame <<- frame+1
        }

	beginFun <- function(c) {
            if (first) { first <<- FALSE; w <<- c[1]; h <<- c[2]; setup.FUN(c[1],c[2]) }
        }

    voluntary <- FALSE
    while (.Call("MP_read", format, beginFun, dumpFun, PACKAGE="videoTools")) {
      if (frames>0 && frame>frames) { voluntary <- TRUE; break }
    }

    if (!voluntary || close) .Call("MP_close", PACKAGE="videoTools")
    invisible(frame-1)
}

next.frame <- function(expand=TRUE, timeout=10) {
  l <- list()
  mplayer(resume=TRUE, close=FALSE, frames=1, expand=expand, timeout=timeout, FUN=function(...) l<<-list(...))
  l
}
