\name{display}
\alias{new.display}
\alias{update.videodisplay}
\alias{close.videodisplay}
\alias{==.videodisplay}
\alias{print.videodisplay}
\title{
  Create, update or close a video display
}
\description{
  \code{new.display} creates a new video display

  \code{update.videodisplay} updates the contents of a video display

  \code{close.videodisplay} closes a video display
}
\usage{
new.display(w, h, rgb = TRUE, ...)
\method{update}{videodisplay}(object, r, g, b, ...)
\method{close}{videodisplay}(con, ...)
\method{==}{videodisplay}(e1, e2)
\method{print}{videodisplay}(x, ...)
}
\arguments{
  \item{w}{width of the video (in pixels)}
  \item{h}{height of the video (in pixels)}
  \item{rgb}{determines the image format (RGB or YUV)}
  \item{r}{red or luminance (Y) channel as a matrix or vector}
  \item{g}{green, chroma combined (UV) or separate U channel}
  \item{b}{blue or chroma (V) channel}
  \item{e1}{display to compare}
  \item{e2}{display to compare}
  \item{x}{display to print}
  \item{...}{any other arguments}
  \item{object}{display object (as returned by \code{new.display})}
  \item{con}{display object (named so for compatibility with the generic)}
}
\value{
  \code{NULL}
}
\details{
  \code{new.display} creates a new video output display and returns the
  associated display object. The result is also assigned into
  \code{.lastDisplay} variable.

  If \code{rgb} is \code{TRUE} then the image format is RGB, otherwise
  it is 4:2:2 YUV (aka YCbCr).

  \code{update} updates the contents of the video display. If
  the image type is RGB then only RGB or Y (monochrome) updates are
  allowed. If the image type is YUV then only YUV updates (with separate
  Y,U,V or combined chroma Y,UV) are allowed. The matrices should have
  the dimensions width,height (except for separate U and V which should
  have dimensions width/2,height).

  \code{close} closes the specified display. If it was also the
  same display as denoted by \code{.lastDisplay} then the
  \code{.lastDisplay} variable is removed (even if other displays are
  open).
}
%\seealso{
%}
\examples{
\dontrun{
d <- new.display(768, 576) # PAL
update(d, matrix(runif(768*576),768))
close(d)
}
}
\keyword{interface}
