\name{mplayer}
\alias{mplayer}
\alias{run.mplayer}
\alias{next.frame}
\title{
  Mplayer interface
}
\description{
  \code{mplayer} runs mplayer and analyzes each frame

  \code{next.frame} fetches next frame from an active mplayer session
  
  \code{run.mplayer} runs mplayer
}
\usage{
mplayer(input, FUN, setup.FUN=NULL, format="RGB24", expand=TRUE, port=7922,
        unix.socket=NULL, save.env=NULL, frames=-1, timeout=10,
        close = TRUE, resume = FALSE)
next.frame(expand = TRUE, timeout = 10)
run.mplayer(file)
}
\arguments{
  \item{input}{input as passed to mplayer on the command line (file,
  tv:// or dvd:// etc.) Note that spaces in filenames must be
  escaped. It can also contain any additional parameters to mplayer.
  If not specified, mplayer is expected to be run manually.}
  \item{FUN}{function to call for each frame}
  \item{setup.FUN}{function to call before first frame (or NULL if not needed)}
  \item{format}{format to use for decoded frames (fourcc integer value
    or string)}
  \item{expand}{for 4:1:1 formats whether to expand chroma matrices to
    full size}
  \item{port}{port to listen on (ignored if unix.socket is used)}
  \item{unix.socket}{name of the unix socket to use or NULL if TCP/IP
    is to be used. As the name implies unix sockets don't work on Windows.}
  \item{save.env}{store internal results to the environment (a list
    named \code{frame} is created with all frames)}
  \item{frames}{fetch only the specified number of frames (-1 means all
    until the end of stream)}
  \item{timeout}{timeout for the accept command (in seconds,
  fractional numbers accepted). Effectively it specifies how long R
  will wait for mplayer to start the playback.}
  \item{close}{when set to \code{FALSE} then the connection will not be
    closed after reaching the frame limit and can be resumed later}
  \item{resume}{resume an existing session, i.e. fetch next frames. When
    set to \code{TRUE} all input-related arguments are ignored}
  \item{file}{file to play back using videoTools vo}
}
\value{
  \code{TRUE}
}
\details{
  \code{mplayer} creates a server that listens in anticipation of a
  connection from the videoTools mplayer module. If input is specified
  then mplayer is launched (in background) with the correct parameters
  to feed this server with frames. Once the connection is
  established, \code{setup.FUN} is called as \code{setup.FUN(width,
    height)} with the frame dimensions. Then each frame received from
  mplayer is decoded and stored in matrix form according to the format
  used. This matrix form is then passed for the \code{FUN}
  function. Depending on the format the calls are as follows:

  \code{RGB} or \code{BGR}: \code{FUN(r, g, b)}

  \code{YV12} (4:1:1): \code{FUN(Y, U, V)}

  \code{YUY2} (4:2:2): \code{FUN(Y, UV)}

  where \code{r}, \code{g}, \code{b} and \code{Y} are width*height
  matrices, \code{U} (aka Cb) and \code{V} (aka Cr) are width/2*height/2
  matrices (unless \code{expand=TRUE}). \code{UV} is a width*height
  matrix with horizontally interlaced \code{U} and \code{V} (thus
  \code{U=UV[1:(dim(U)[1]/2-1)*2-1,]})
  
  Other formats can be used, but they are not decoded to matrix
  form. (Note that \code{I420}=\code{IYUV} and is the same as
  \code{YV12} except for the order of \code{U} and \code{V}. Also
  \code{YUYV}=\code{YUY2})

  Note: All values (including \code{Y}, \code{U} and \code{V}) are
  expanded based on the full 0-255 range of each byte. Correction of
  actually used ranges is currently not performed, but may possibly be
  at a later point.

  \code{run.mplayer} runs the mplayer with correct arguments to connect
  to a local videoTools server.

  Caveat: the \code{timeout} parameter is now silently discarded by the current implementation and thus the accept command waits forever. This should be fixed in the next release.
}
%\seealso{
%}
\examples{
\dontrun{
mplayer("dvd://1", update.display, new.display)
}
}
\keyword{interface}
