\name{rawtools}
\alias{raw2double}
\alias{rawi2double}
\alias{rawi42double}
\alias{rawi32double}
\title{
  Tools for converting raw data into vectors or matrices
}
\description{
  \code{raw2double} converts 8-bit values into a vector or matrix.

  \code{rawi2double} converts interlaced 8-bit values into multiple
  vectors or matrices.

  \code{rawi42double} and \code{rawi32double} are convenience wrappers
  to \code{rawi2double} for commonly used 3 or 4 components.
}
\usage{
raw2double(rv, offset=0, length=-1, stride=0, x.expand=1, y.expand=1)
rawi2double(rv, offset=0, length=-1, width=0, channels=4)
rawi42double(rv, offset=0, length=-1, width=0)
rawi32double(rv, offset=0, length=-1, width=0)
}
\arguments{
  \item{rv}{raw vector}
  \item{offset}{offset in the raw vector at which to start decoding}
  \item{length}{number of bytes to decode (or -1 for all available)}
  \item{stride}{number of bytes that compose a matrix column (or 0 if the result
    should be a vector)}
  \item{channels}{number of interlaced channels}
  \item{width}{width of the resulting video frame}
  \item{x.expand}{repeat each matrix row that many times (stride must be
    specified if != 1)}
  \item{y.expand}{repeat each matrix column that many times (stride must
    be specified if != 1)}
}
\value{
  \code{raw2double}: single vector (if stride<1) or a single matrix (if
  stride>=1)

  \code{rawi2double}: list (of length \code{channels}) of vectors (if width<1) or matrices (if width>=1)
}
\details{
  \code{raw2double} converts a part of a raw vector into a real vector
  or matrix. The byte range 0..255 is mapped into 0..1. The result is a
  matrix if the \code{stride} is specified. Optionally, the resulting
  matrix can be expanded (e.g. use \code{x.expand=2,y.expand=2} to
  convert 4:1:1 chroma into full 4:4:4, or \code{x.expand=2,y.expand=1} to
  convert 4:2:2 chroma into full 4:4:4).

  \code{rawi2double} converts a part of a raw vector into a list of real
  vectors or matrices. The values are assumed to be interlaced at byte
  boundary (e.g. \code{ABCABC..} would be 3 channels resulting in three
  matrices corresponding to \code{AA..}, \code{BB..} and \code{CC..}).
}
%\seealso{
%}
%\examples{
%\dontrun{
%}
%}
\keyword{interface}
