#if !defined WIN32 && !defined unix
#define unix
#endif

#define MAIN
#include "sisocks.h"

#ifndef unix
typedef int socklen_t;
#define CAN_TCP_NODELAY
#endif

#ifdef unix
#include <sys/un.h> /* needed for unix sockets */
#include <stdio.h>  /* needed for rmove to delete the socket */
#endif

#define IPC_CMD_HELO 0x00000100
#define IPC_CMD_OLEH 0x00010000
#define IPC_CMD_HELO 0x00000100

#define IPC_CMD_ACCEPT_FORMAT 0x01
#define IPC_CMD_DUMP_FRAME    0x02
#define IPC_CMD_CLOSE         0x03
#define IPC_CMD_BEGIN         0x04

#define IPC_RES_YES  0x00010100
#define IPC_RES_NO   0x00020200

static SOCKET s = -1, ss = -1, as = -1;

static int ipc_connect(const char *host, int port) {
#ifdef unix
  struct sockaddr_un sau;
#endif
  SAIN sai;
  int i;

  if (port) {
    memset(&sai,0,sizeof(sai));
    build_sin(&sai,host,port);
  } else {
#ifdef unix
    memset(&sau,0,sizeof(sau));
    sau.sun_family=AF_LOCAL;
    strcpy(sau.sun_path,host); /* FIXME: possible overflow! */
#else
    return -11;  /* unsupported */
#endif
  }
  s = socket(port?AF_INET:AF_LOCAL, SOCK_STREAM, 0);
  if (port) {
#ifdef CAN_TCP_NODELAY
    int opt=1;
    setsockopt(s, IPPROTO_TCP, TCP_NODELAY, (const char*) &opt, sizeof(opt));
#endif
    i=connect(s, (SA*)&sai, sizeof(sai));
  }
#ifdef unix
  else
    i=connect(s, (SA*)&sau, sizeof(sau));
#endif
  if (i==-1) {
    closesocket(s); s=-1;
    return -1; /* connect failed */
  }
  return 0;
}

static int ipc_bind(const char *host, int port) {
  SAIN ssa;
#ifdef unix
  struct sockaddr_un lusa;
#endif
  int reuse = 1, i;
  if (ss!=-1) { closesocket(ss); ss=-1; }
#ifndef unix
  if (!port) return -11; /* unsupported */
#endif
  ss = socket(port?AF_INET:AF_LOCAL, SOCK_STREAM, 0);
  if (port) {
    setsockopt(ss,SOL_SOCKET,SO_REUSEADDR,&reuse,sizeof(reuse));
    i = bind(ss, build_sin(&ssa,0,port),sizeof(ssa));
    if (!i) i = listen(ss, 64);
    return i;
  }
  memset(&lusa,0,sizeof(lusa));
  lusa.sun_family=AF_LOCAL;
  if (strlen(host)>sizeof(lusa.sun_path)-2) return -11;
  strcpy(lusa.sun_path,host);
  remove(host);
  i = bind(ss,(SA*) &lusa, sizeof(lusa));
  if (!i) i = listen(ss, 64);
  return i;
}

static int ipc_accept(double timeout) {
  char abuf[512];
  int abl = 512;
  if (ss == -1) return -1;
  if (as != -1) { closesocket(as); as=-1; }
  as = accept(ss, (SA*) abuf, &abl);
  return (as == -1)?-1:0;
}

static void ipc_close() {
  if (s>-1) {
    closesocket(s);
    s=-1;
  }
}

struct s_header {
  int cmd;
  int len;
};

static int ipc_send_hdr(int cmd, int len) {
  if (s<0) return -1;
  {
    struct s_header h = { cmd, len };
    if (send(s, (char*)&h, 8, 0) != 8) { ipc_close(); return -2; }
  }
  return 0;
}

static int ipc_send_data(char *buf, int len) {
  if (s<0) return -1;
  if (len>0) {
    if (send(s, (char*) buf, len, 0) != len) { ipc_close(); return -2; }
  }
  return 0;
}

static int ipc_send(int cmd, void *buf, int len) {
  int i = ipc_send_hdr(cmd, len);
  if (i==0 && len>0) i=ipc_send_data(buf, len);
  return i;
}

static int ipc_resp() {
  int i;
  if (s<0) return -1;
  if (recv(s, (void*)&i, 4, 0)!=4) { ipc_close(); return -1; }
  return i;
}

static int ipc_closec() {
  if (as!=-1) closesocket(as);
  as=-1;
  return 0;
}

static int ipc_closes() {
  if (ss!=-1) closesocket(ss);
  ss=-1;
  return 0;
}

static int ipc_send_resp(int r) {
  if (as < 0) return -1;
  if (send(as, &r, 4, 0) != 4) { ipc_closec(); return -1; }
  return 0;
}

static int rh[2];

static int* ipc_recv_hdr() {
  if (as<0) return 0;
  if (recv(as, (void*)rh, 8, 0)!=8) { ipc_closec(); return 0; }
  return rh;
}

static int ipc_recv_data(void *buf, unsigned int len) {
  int n;
  if (as<0) return -1;
  while (len > 0) {
    n = recv(as, buf, len, 0);
    if (n < 1) { ipc_closec(); return -2; }
    buf += n; len -= n;
  }
  return 0;
}

/* mplayer callbacks */

static void vo_connect() {
  const char *host="127.0.0.1", *e;
  int port = 7922;
  
  e = getenv("VOSOCKET");
  if (e && *e) {
    port = 0; host = e;
  } else {
    e = getenv("VOHOST");
    if (e && *e) host = e;
    e = getenv("VOPORT");
    if (e && *e) port = atoi(e);
  }

  /*fprintf(stderr,"attemptin connect %s:%d\n", host, port);*/
  ipc_connect(host, port);
}

int vo_accept_format(unsigned int format) {
  if (s<0) vo_connect();
  if (ipc_send(IPC_CMD_ACCEPT_FORMAT, &format, 4)) return 0;
  return (ipc_resp()==IPC_RES_YES)?1:0;
}

int vo_dump_frame(void *buf, int w, int h, int f, int chs, int flags) {
  int fr[5] = { w, h, f, chs, flags };
  int l = sizeof(fr) + w*h*chs;
  int r;
  r=ipc_send_hdr(IPC_CMD_DUMP_FRAME, l);
  if (!r) r=ipc_send_data((void*)fr, sizeof(fr));
  if (!r) r=ipc_send_data(buf, l-sizeof(fr));
  return r?-1:0;
}

int  vo_begin(int w, int h, int fmt) {
  int a[3] = { w, h, fmt };
  return ipc_send(IPC_CMD_BEGIN, &a, 3*4)?-1:0;
}

void vo_end() {
  ipc_send(IPC_CMD_CLOSE, 0, 0);
  ipc_close();
}

/* R side */

#include <R.h>
#include <Rinternals.h>

SEXP MP_bind(SEXP host, SEXP port) {
  int p = asInteger(port);
  char *h = p?0:CHAR(asChar(host));
  int i = ipc_bind(h, p);
  if (i) error("unable to listen for mplayer");
  return R_NilValue;
}


SEXP MP_accept(SEXP timeout) {
  double d = asReal(timeout);
  SEXP r = allocVector(LGLSXP, 1);
  LOGICAL(r)[0] = (ipc_accept(d))?0:1;
  return r;
}

SEXP MP_active() {
  return ScalarLogical((as==-1)?0:1);
}

SEXP MP_read(SEXP format, SEXP beginFun, SEXP dumpFun) {
  int *h;
  int desired_format = asInteger(format);

  h = ipc_recv_hdr();
  if (!h) error("communication error");
  
  switch (h[0]) {
  case IPC_CMD_ACCEPT_FORMAT:
    {
      int f;
      if (ipc_recv_data(&f, 4)) {
	ipc_closec(); ipc_closes();
	error("communication error (in accept_format)");
      }
      if (ipc_send_resp((f == desired_format)?IPC_RES_YES:IPC_RES_NO)) {
	ipc_closes();
	error("communication error (in accept_format)");
      }
    }
    break;
  case IPC_CMD_DUMP_FRAME:
    {
      int dfh[5];
      SEXP r, iv;
      if (ipc_recv_data(dfh, 4*5)) {
        ipc_closec(); ipc_closes();
        error("communication error (in dump_frame)");
      }
      r = allocVector(RAWSXP, h[1] - 4*5);
      if (ipc_recv_data(RAW(r), LENGTH(r))) {
        ipc_closec(); ipc_closes();
        error("communication error (in dump_frame, fetching frame data)");
      }
      PROTECT(r);
      PROTECT(iv = allocVector(INTSXP, 5));
      memcpy(INTEGER(iv), dfh, 5*4);
      r = eval(LCONS(dumpFun, LCONS(iv, LCONS(r, R_NilValue))), CLOENV(dumpFun));
      UNPROTECT(2);
      if (asInteger(r)==-1) {
	ipc_closec(); ipc_closes();
	break;
      }
    }
    break;
    
  case IPC_CMD_BEGIN:
    {
      int bh[h[1]/4+1];
      SEXP iv;
      if (ipc_recv_data(bh, h[1])) {
        ipc_closec(); ipc_closes();
        error("communication error (in begin)");
      }
      iv = allocVector(INTSXP, h[1]/4);
      memcpy(INTEGER(iv), bh, (h[1]/4) * 4);
      iv = eval(LCONS(beginFun, LCONS(iv, R_NilValue)), CLOENV(beginFun));
      if (asInteger(iv) == -1) {
	ipc_closec(); ipc_closes();
	break;
      }
    }
    break;

  case IPC_CMD_CLOSE:
    ipc_closec(); ipc_closes();
    break;

  default:
    ipc_closec(); ipc_closes();
    error("unknown command from mplayer: %d, aborting", h[0]);
  }

  return MP_active();
}

SEXP MP_close() {
  ipc_closec(); ipc_closes();
  return R_NilValue;
}
