#include <R.h>
#include <Rinternals.h>

SEXP raw_to_double(SEXP rv, SEXP off, SEXP len, SEXP stride, SEXP exx, SEXP exy) {
  int o = asInteger(off);
  int l = asInteger(len);
  int ex = asInteger(exx), ey = asInteger(exy), os = asInteger(stride);
  int i = 0;
  unsigned char *c;
  double *d;
  SEXP nv;
  if (TYPEOF(rv) != RAWSXP)
    error("first argument must be a raw vector");
  if (o >= LENGTH(rv)) { o = 0; l = 0; }
  if (l < 0) l = LENGTH(rv) - o;
  if (ex<2 && ey<2) {
    nv = allocVector(REALSXP, l);
    d = REAL(nv);
    c = (unsigned char*) RAW(rv); c+=o;
    while (i<l) {
      d[i] = ((double)c[i])/255.0; i++;
    }
    if (os>0) {
      PROTECT(nv);
      {
	SEXP dv = allocVector(INTSXP, 2);
	INTEGER(dv)[0] = os;
	INTEGER(dv)[1] = l/os;
	setAttrib(nv, R_DimSymbol, dv);
      }
      UNPROTECT(1);
    }
    return nv;
  }
  if (os<1) error("stride must be supplied if expansion is to be used");
  {
    int h = l/os;
    int y = 0;
    double *cd;
    PROTECT(nv = allocVector(REALSXP, os*ex*h*ey));
    cd = d = REAL(nv);
    c = (unsigned char*) RAW(rv); c+=o;
    while(y < h) {
      int x = 0;
      while (x < os) {
	if (ex < 1) {
	  *cd = ((double)*c)/255.0; cd++;
	} else {
	  i=0; while (i<ex) { *cd = ((double)*c)/255.0; cd++; i++; }
	}
	c++;
	x++;
      }
      if (ey>1) {
	i=1; while (i<ey) {
	  memcpy(cd, cd-(os*ex), os*ex*sizeof(double));
	  cd+=(os*ex); i++;
	}
      }
      y++;
    }
    {
      SEXP dv = allocVector(INTSXP, 2);
      INTEGER(dv)[0] = os*ex;
      INTEGER(dv)[1] = l/os*ey;
      setAttrib(nv, R_DimSymbol, dv);
    }
    UNPROTECT(1);
  }
  return nv;
}

SEXP rawi_to_double(SEXP rv, SEXP off, SEXP len, SEXP width, SEXP channels) {
  int o = asInteger(off);
  int l = asInteger(len);
  int i = 0, w = asInteger(width), chs = asInteger(channels);
  unsigned char *c;
  double *d[8];
  SEXP nv;
  if (TYPEOF(rv) != RAWSXP)
    error("first argument must be a raw vector");
  if (chs<1 || chs>8)
    error("number of channels must be between 1 and 8");
  if (o >= LENGTH(rv)) { o = 0; l = 0; }
  if (l < 0) l = LENGTH(rv) - o;
  l/=chs;
  PROTECT(nv=allocVector(VECSXP, chs));
  while (i < chs) {
    SEXP v = allocVector(REALSXP, l);
    if (w>0) {
      PROTECT(v);
      {
	SEXP dv = allocVector(INTSXP, 2);
	INTEGER(dv)[0] = w;
	INTEGER(dv)[1] = l/w;
	setAttrib(v, R_DimSymbol, dv);
      }
      SET_VECTOR_ELT(nv, i, v);
      UNPROTECT(1);
    } else
      SET_VECTOR_ELT(nv, i, v);
    d[i] = REAL(v);
    i++;
  }
  i = 0;
  c = (unsigned char*) RAW(rv); c+=o;
  while (i < l) {
    unsigned int cn = 0;
    while (cn < chs)
      { d[cn++][i] = ((double)*c)/255.0; c++; }
    i++;
  }

  UNPROTECT(1);
  return nv;
}

SEXP raw_to_int(SEXP rv, SEXP off, SEXP len) {
  int o = asInteger(off);
  int l = asInteger(len);
  int i = 0;
  unsigned char *c;
  int *d;
  SEXP nv;
  if (TYPEOF(rv) != RAWSXP)
    error("first argument must be a raw vector");
  if (o >= LENGTH(rv)) { o = 0; l = 0; }
  if (l < 0) l = LENGTH(rv) - o;
  nv = allocVector(INTSXP, l);
  d = INTEGER(nv);
  c = (unsigned char*) RAW(rv); c+=o;
  while (i<l) {
    d[i] = (int) c[i]; i++;
  }
  return nv;
}

SEXP fourcc2int(SEXP fcc) {
  SEXP r;
  unsigned char *c = (unsigned char*) CHAR(asChar(fcc));
  unsigned int i = 0, l = 0;
  while (*c && l<4) {
    i=(((unsigned int)*c)<<24)|(i>>8);
    c++; l++;
  }
  /* some special cases - RGB/BGR are not strictly fourcc */
  if (i==0x33424752 && *c=='2') i=0x42475220; /* RGB32 */
  if (i==0x32424752 && *c=='4') i=0x42475218; /* RGB24 */
  if (i==0x31424752 && *c=='6') i=0x42475210; /* RGB16 */
  if (i==0x31524742 && *c=='6') i=0x52474210; /* BGR16 */
  if (i==0x32524742 && *c=='4') i=0x52474218; /* BGR24 */
  if (i==0x33524742 && *c=='2') i=0x52474220; /* BGR32 */
  r=allocVector(INTSXP, 1); INTEGER(r)[0] = i;
  return r;
}

SEXP int2fourcc(SEXP iv) {
  int i = asInteger(iv);
  unsigned char fcc[6], *c=fcc;
  while (i) {
    *c = i&255; i>>=8; c++;
  }
  if (i==0x42475220) return mkString("RGB32");
  if (i==0x42475218) return mkString("RGB24");
  if (i==0x42475210) return mkString("RGB16");
  if (i==0x52474210) return mkString("BGR16");
  if (i==0x52474218) return mkString("BGR24");
  if (i==0x52474220) return mkString("BGR32");
  *c=0;
  return mkString((char*)fcc);
}
